module Escalator

  class Command

    class Archive < Command

      class Help < CLAide::Help

        def message
          [
            formatted_error_message,
            "请执行: escalator archive --help".ansi.green
          ].compact.join("\n\n").insert 0, "\n"
        end

      end

      attr_reader :method

      attr_reader :project_path

      attr_reader :output_path

      attr_reader :show_output
      alias_method :show_output?, :show_output

      self.command = "archive"

      self.summary = "打包 Xcode 工程"

      self.description = <<-DESC
      利用 Xcode 工程的配置的编译选项, 和自定义参数, 自动打包 ipa 并导出

      `METHOD` 为 ipa 包导出类型: #{"app-store".ansi.green}, #{"ad-hoc".ansi.green}
      DESC

      self.arguments = [
        CLAide::Argument.new('METHOD', true, false)
      ]

      def self.options
        [
          ["--project-path=path", "path 为 .xcproject 源路径"],
          ["--output-path=path",  "path 为 .ipa 以及部分重签资源的存放路径"],
          ["--show-output",       "文件管理中展示输出文件夹, 默认: 否"]
        ].concat super
      end

      def initialize argv
        @method = argv.arguments!.first
        @project_path = argv.option "project-path", ""
        @output_path = argv.option "output-path", ""
        @show_output = argv.flag? "show-output", false
        super
      end

      def validate!
        super
        if !method
          help! "未检测到 METHOD 参数"
        elsif !%w(app-store ad-hoc).include? method
          help! "METHOD 参数只能为 #{"app-store".ansi.green}, #{"ad-hoc".ansi.green}"
        end
        if project_path.empty?
          help! "未检测到 --project-path 参数"
        end
        if output_path.empty?
          help! "未检测到 --output-path 参数"
        end
      end

      def help!(error_message = nil)
        invoked_command_class.help!(error_message, Help)
      end

      def run
        Escalator::Archive.run self
      end

    end

  end

end
