module Escalator

  class Command

    class Combine < Command

      class Help < CLAide::Help

        def message
          [
            formatted_error_message,
            "请执行: escalator combine --help".ansi.green
          ].compact.join("\n\n").insert 0, "\n"
        end

      end

      attr_reader :commands

      attr_reader :config_path

      attr_reader :input_path

      attr_reader :work_path

      self.command = "combine"

      self.summary = "组合 命令: confuse archive resign upload"

      self.description = <<-DESC
      可选择组合相关命令, 创建执行自定义任务流

      `COMMAND` 要组合的命令, 支持 #{"confuse archive resign upload".ansi.green} 命令, 进行选择组合执行

      注意命令的功能之间, 是否具备组合粘性:

          #{"上一条命令的输出预期 => 下一条命令的输入预期 匹配".ansi.yellow}

      `--input-path=path` 参数下目录结构要求:

      1. #{"confuse archive".ansi.green} 为初始命令

          --input-path
                |
                | --- 工程名 (支持多个)
                |       |
                | ----- | --- .xcodeproj 文件
                |       |
                | ----- | --- CoverInfo.plist 文件 (可缺省: 执行 #{"escalator setup".ansi.green} 获取配置模版)

      2. #{"resign upload".ansi.green} 为初始命令

          --input-path
                |
                | --- 工程名 (支持多个)
                |       |
                | ----- | --- .ipa 文件
                |       |         |
                | ----- | --- .xcassets 文件 (ipa 对应工程下所有被引用的的 .xcassets)
                |       |         |
                | ----- | --- CoverInfo.plist 文件 (可缺省: 执行 #{"escalator setup".ansi.green} 获取配置模版)
      DESC

      self.arguments = [
        CLAide::Argument.new('COMMAND', true, true)
      ]

      def self.options
        [
          ["--config-path=path", "path 为任务流需要的配置参数 ConfigInfo.plist 文件路径, 执行 #{"escalator setup".ansi.green} 获取配置模版"],
          ["--input-path=path", "path 为初始命令需要的输入根目录, 会自动检测需要的输入文件"],
          ["--work-path=path", "path 为脚本工作的根目录"]
        ].concat super
      end

      def initialize argv
        @commands = argv.arguments!
        @config_path = argv.option "config-path", ""
        @input_path = argv.option "input-path", ""
        @work_path = argv.option "work-path", ""
        super
      end

      def validate!
        super
        if commands.empty?
          help! "未检测到 COMMAND 参数"
        end
        cmds = (commands - (%w(confuse archive resign upload) & commands))
        if !cmds.empty?
          help! "检测到 非法参数: #{cmds.join ", "}"
        end
        if config_path.empty?
          help! "未检测到 --config-path 参数"
        end
        if input_path.empty?
          help! "未检测到 --input-path 参数"
        end
        if work_path.empty?
          help! "未检测到 --work-path 参数"
        end
      end

      def help!(error_message = nil)
        invoked_command_class.help!(error_message, Help)
      end

      def run
        Escalator::Combine.run self
      end

    end

  end

end
