module Escalator

  class Command

    class Confuse < Command

      class Help < CLAide::Help

        def message
          [
            formatted_error_message,
            "请执行: escalator confuse --help".ansi.green
          ].compact.join("\n\n").insert 0, "\n"
        end

      end

      attr_reader :keywords

      attr_reader :project_path

      attr_reader :output_path

      attr_reader :show_output
      alias_method :show_output?, :show_output

      self.command = "confuse"

      self.summary = "混淆 Xcode 工程"

      self.description = <<-DESC
      使用置换关键词的方式混淆项目内容和文件名

      `KEYWORD` 要置换的关键词, 关键词个数不限

      支持混淆的文件类型: #{"h, hpp, pch, c, m, mm, cpp, swift, xib, storyboard".ansi.yellow}

      支持 #{"Swift".ansi.yellow}, #{"OC".ansi.yellow} 混编模式
      DESC

      self.arguments = [
        CLAide::Argument.new('KEYWORD', true, true)
      ]

      def self.options
        [
          ["--project-path=path", "path 为 .xcodeproj 源路径"],
          ["--output-path=path",  "path 为混淆工程存放路径"],
          ["--show-output",       "文件管理中展示输出文件夹, 默认: 否"]
        ].concat super
      end

      def initialize argv
        @keywords = argv.arguments!.select { |arg| !arg.empty? }
        @project_path = argv.option "project-path", ""
        @output_path = argv.option "output-path", ""
        @show_output = argv.flag? "show-output", false
        super
      end

      def validate!
        super
        if keywords.empty?
          help! "未检测到 KEYWORD 参数"
        end
        if project_path.empty?
          help! "未检测到 --project-path 参数"
        end
        if output_path.empty?
          help! "未检测到 --output-path 参数"
        end
      end

      def help!(error_message = nil)
        invoked_command_class.help!(error_message, Help)
      end

      def run
        Escalator::Confuse.run self
      end

    end

  end

end
