module Escalator

  class Command

    class Resign < Command

      class Help < CLAide::Help

        def message
          [
            formatted_error_message,
            "请执行: escalator resign --help".ansi.green
          ].compact.join("\n\n").insert 0, "\n"
        end

      end

      attr_reader :ipaPaths

      attr_reader :key_id

      attr_reader :issuer_id

      attr_reader :keyfile_path

      attr_reader :apple_id

      attr_reader :user_password

      attr_reader :bundle_identifier

      attr_reader :short_version

      attr_reader :bundle_version

      attr_reader :bundle_name

      attr_reader :display_name

      attr_reader :executable_name

      attr_reader :iconAssets_path

      self.command = "resign"

      self.summary = "重签 IPA 安装包"

      self.description = <<-DESC
      利用新的 #{"证书".ansi.yellow} 和 #{"配置".ansi.yellow} 对当前 IPA 安装包进行重签名并修改相应信息

      Appstore Connect 用户访问密钥申请:

          #{"https://appstoreconnect.apple.com => 登录 => 用户和访问 => 密钥".ansi.blue}

      `IPAPATH` 要重签名的 IPA 路径, 支持同一账号处理多个 IPA

      `--iconAssets-path=path` 参数若存在, 务必将 ipa 对应 Xcode 工程下所有被引用的 .xcassets 文件的拷贝到 IPA 所在的目录下, 若不提供 APP 将丢失所有老资源
      DESC

      self.arguments = [
        CLAide::Argument.new('IPAPATH', true, false)
      ]

      def self.options
        [
          ["--key-id=id", "id 为 Appstore Connect 用户访问密钥的 keyId"],
          ["--issuer-id=id", "id 为 Appstore Connect 用户访问密钥的 issuerId"],
          ["--keyfile-path=path", "path 为 Appstore Connect 用户访问密钥的 .p8 文件保存路径"],
          ["--user-password=password", "password 为当前电脑用户的访问密码"],
          ["--bundle-identifier=identifier", "identifier 为新的唯一标识符"],
          ["--short-version=version", "version 为新的版本号"],
          ["--bundle-version=version", "version 为新的 BuildID, 同一账号下处理多 IPA, 会自动叠加"],
          ["--apple-id=id", "id 为对应的苹果开发者账户, 如果含有 APP GROUP 推荐添加此字段, 默认: 无"],
          ["--bundle-name=name", "name 为新的包名, 默认: 不变"],
          ["--display-name=name", "name 为新的 APP 名称, 默认: 不变"],
          ["--executable-name=name", "name 为新的二进制文件名, 默认: 不变"],
          ["--iconAssets-path=path", "path 为包含新 AppIcon 的 .xcassets 资源文件路径, 默认: 无"]
        ].concat super
      end

      def initialize argv
        @ipaPaths = argv.arguments!.select{ |arg| !arg.empty? }
        @key_id = argv.option "key-id", ""
        @issuer_id = argv.option "issuer-id", ""
        @keyfile_path = argv.option "keyfile-path", ""
        @apple_id = argv.option "apple-id", ""
        @user_password = argv.option "user-password", ""
        @bundle_identifier = argv.option "bundle-identifier", ""
        @short_version = argv.option "short-version", ""
        @bundle_version = argv.option "bundle-version", ""
        @bundle_name = argv.option "bundle-name", ""
        @display_name = argv.option "display-name", ""
        @executable_name = argv.option "executable-name", ""
        @iconAssets_path = argv.option "iconAssets-path", ""
        super
      end

      def validate!
        super
        if ipaPaths.empty?
          help! "未检测到 IPAPATH 参数"
        end
        if key_id.empty?
          help! "未检测到 --key-id 参数"
        end
        if issuer_id.empty?
          help! "未检测到 --issuer-id 参数"
        end
        if keyfile_path.empty?
          help! "未检测到 --keyfile-path 参数"
        end
        if user_password.empty?
          help! "未检测到 --user-password 参数"
        end
        if bundle_identifier.empty?
          help! "未检测到 --bundle-identifier 参数"
        end
        if short_version.empty?
          help! "未检测到 --short-version 参数"
        end
        if bundle_version.empty?
          help! "未检测到 --bundle-version 参数"
        end
      end

      def help!(error_message = nil)
        invoked_command_class.help!(error_message, Help)
      end

      def run
        Escalator::Resign.run self
      end

    end

  end

end
