module Escalator

  class Command

    class Setup < Command

      class Help < CLAide::Help

        def message
          [
            formatted_error_message,
            "请执行: escalator setup --help".ansi.green
          ].compact.join("\n\n").insert 0, "\n"
        end

      end

      attr_reader :commands

      attr_reader :output_path

      attr_reader :set_zshcompletion

      attr_reader :custom_plugins

      self.command = "setup"

      self.summary = "生成 组合命令执行需要的参数模版"

      self.description = <<-DESC
      导出组合命令执行必须需的, 全局配置参数模版 和 独立配置参数模版

      独立参数模版, 在包含 #{"resign".ansi.green} 命令时才会导出

      独立参数模版, 需放置在对应输入项目的根路径下, 会覆盖全局配置中对应的参数

      `COMMAND` 要组合的命令, 支持 #{"confuse archive resign upload".ansi.green} 命令
      DESC

      self.arguments = [
        CLAide::Argument.new('COMMAND', true, true)
      ]

      def self.options
        [
          ["--output-path=path", "path 为参数模版导出路径"],
          ["--set-zshcompletion", "会忽略其他参数, 只设置 oh-my-zsh 的命令补全插件到 custom/plugins 中, zshrc 配置插件, 请自己手动更新, 默认: 否"]
        ].concat super
      end

      def initialize argv
        @commands = argv.arguments!.uniq
        @output_path = argv.option "output-path"
        @set_zshcompletion = argv.flag? "set-zshcompletion", false
        super
      end

      def validate!
        super
        if set_zshcompletion
          if ENV["ZSH"].empty?
            help! "未检测到 oh-my-zsh, 请先进行安装"
          end
          @custom_plugins = "#{ENV["ZSH"]}/custom/plugins/"
          return
        end
        if commands.empty?
          help! "未检测到 COMMAND 参数"
        end
        cmds = (commands - (%w(confuse archive resign upload) & commands))
        if !cmds.empty?
          help! "检测到 非法参数: #{cmds.join ", "}"
        end
        if !output_path
          help! "未检测到 --output-path 参数"
        end
      end

      def help!(error_message = nil)
        invoked_command_class.help!(error_message, Help)
      end

      def run
        Escalator::Setup.run self
      end

    end

  end

end
