module Escalator

  class Command

    class Upload < Command

      class Help < CLAide::Help

        def message
          [
            formatted_error_message,
            "请执行: escalator upload --help".ansi.green
          ].compact.join("\n\n").insert 0, "\n"
        end

      end

      attr_reader :ipaPaths

      attr_reader :key_id

      attr_reader :issuer_id

      attr_reader :keyfile_path

      attr_reader :bundle_identifier

      self.command = "upload"

      self.summary = "上传 IPA 安装包"

      self.description = <<-DESC
      利用 Appstore Connect 用户访问密钥, 上传 ipa 安装包

      Appstore Connect 用户访问密钥申请:

          #{"https://appstoreconnect.apple.com => 登录 => 用户和访问 => 密钥".ansi.blue}

      `IPAPATH` 要上传的 IPA 路径, 支持同一账号上传多个 IPA
      DESC

      self.arguments = [
        CLAide::Argument.new('IPAPATH', true, true)
      ]

      def self.options
        [
          ["--key-id=id", "id 为 Appstore Connect 用户访问密钥的 keyId"],
          ["--issuer-id=id", "id 为 Appstore Connect 用户访问密钥的 issuerId"],
          ["--keyfile-path=path", "path 为 Appstore Connect 用户访问密钥的 .p8 文件保存路径"],
          ["--bundle-identifier=identifier", "identifier 为上传账户下对应唯一标识符"]
        ].concat super
      end

      def initialize argv
        @ipaPaths = argv.arguments!.select { |arg| !arg.empty? }
        @key_id = argv.option "key-id", ""
        @issuer_id = argv.option "issuer-id", ""
        @keyfile_path = argv.option "keyfile-path", ""
        @bundle_identifier = argv.option "bundle-identifier", ""
        super
      end

      def validate!
        super
        if ipaPaths.empty?
          help! "未检测到 IPAPATH 参数"
        end
        if key_id.empty?
          help! "未检测到 --key-id 参数"
        end
        if issuer_id.empty?
          help! "未检测到 --issuer-id 参数"
        end
        if keyfile_path.empty?
          help! "未检测到 --keyfile-path 参数"
        end
        if bundle_identifier.empty?
          help! "未检测到 --bundle-identifier 参数"
        end
      end

      def help!(error_message = nil)
        invoked_command_class.help!(error_message, Help)
      end

      def run
        Escalator::Upload.run self
      end

    end

  end

end
